#ifndef MYX_REDIS_CLIENT_INL_HPP_
#define MYX_REDIS_CLIENT_INL_HPP_

#pragma once

#ifndef MYXLIB_HEADER_ONLY
#include <myx/redis/client.hpp>
#endif

namespace myx {

namespace redis {

MYXLIB_INLINE Client::Client( QObject* parent ) :
	QObject( parent ),
	d      ( new ClientPrivate( this ) )
{
}


MYXLIB_INLINE void Client::connectToHost( const QString& hostName, quint16 port )
{
	d->m_socket.connectToHost( hostName, port );
}


MYXLIB_INLINE void Client::disconnectFromHost()
{
	d->m_socket.disconnectFromHost();
}


MYXLIB_INLINE bool Client::isConnected() const
{
	return( d->m_socket.state() == QAbstractSocket::ConnectedState );
}


MYXLIB_INLINE Request* Client::sendCommand( const QByteArray& command )
{
	d->m_socket.write( command + "\r\n" );

	auto* request = new Request( this );
	d->m_queue.enqueue( request );
	return( request );
}


MYXLIB_INLINE bool Client::waitForConnected( int msecs )
{
	return( d->m_socket.waitForConnected( msecs ) );
}


MYXLIB_INLINE bool Client::waitForDisconnected( int msecs )
{
	return( d->m_socket.waitForDisconnected( msecs ) );
}

} // namespace redis

} // namespace myx

#endif // ifndef MYX_REDIS_CLIENT_INL_HPP_
